package com.lantone.security.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.lantone.common.dto.DataAuthDTO;
import com.lantone.common.dto.DataAuthDetailDTO;
import com.lantone.common.dto.DeptDTO;
import com.lantone.common.dto.DoctorDTO;
import com.lantone.common.dto.HospitalDTO;
import com.lantone.common.dto.RoleDTO;
import com.lantone.common.dto.SoftwareRoleDataAuthDetailDTO;
import com.lantone.common.dto.SysServiceDTO;
import com.lantone.common.exception.Asserts;
import com.lantone.common.util.DateUtil;
import com.lantone.common.util.ListUtil;
import com.lantone.common.util.StringUtil;
import com.lantone.common.vo.DataAuthDetailVO;
import com.lantone.common.vo.DataAuthVO;
import com.lantone.common.vo.DoctorVO;
import com.lantone.common.vo.RoleVO;
import com.lantone.common.vo.SoftwareDataAuthVO;
import com.lantone.common.vo.SysSoftwareVO;
import com.lantone.dblayermbg.entity.DataAuth;
import com.lantone.dblayermbg.entity.DataAuthDetail;
import com.lantone.dblayermbg.entity.Dept;
import com.lantone.dblayermbg.entity.Hospital;
import com.lantone.dblayermbg.entity.RoleSoftwareDataAuth;
import com.lantone.dblayermbg.entity.SoftwareDataAuth;
import com.lantone.dblayermbg.facade.DataAuthDetailFacade;
import com.lantone.dblayermbg.facade.DataAuthFacade;
import com.lantone.dblayermbg.facade.DeptFacade;
import com.lantone.dblayermbg.facade.DoctorFacade;
import com.lantone.dblayermbg.facade.HospitalFacade;
import com.lantone.dblayermbg.facade.RoleSoftwareDataAuthFacade;
import com.lantone.dblayermbg.facade.SoftwareDataAuthFacade;
import com.lantone.dblayermbg.service.impl.DataAuthServiceImpl;
import com.lantone.security.enums.CRUDEnum;
import com.lantone.security.enums.DataAuthDataTypeEnum;
import com.lantone.security.enums.DataAuthDetailTypeEnum;
import com.lantone.security.enums.HospitalTypeEnum;
import com.lantone.security.enums.IsDeleteEnum;
import com.lantone.security.enums.ServiceEnum;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;


/**
 * @ClassName: DataAuthHandleFacade
 * @Description:数据权限操作facade
 * @Author songxl
 * @Date 2021/7/21
 * @Version 1.0
 */
@Component
public class DataAuthHandleFacade extends DataAuthServiceImpl {
    @Autowired
    private DataAuthFacade dataAuthFacade;
    @Autowired
    private SoftwareDataAuthFacade softwareDataAuthFacade;
    @Autowired
    private RoleSoftwareDataAuthFacade roleSoftwareDataAuthFacade;
    @Autowired
    private DataAuthDetailFacade dataAuthDetailFacade;
    @Autowired
    private HospitalFacade hospitalFacade;
    @Autowired
    private DeptFacade deptFacade;
    @Autowired
    private DoctorFacade doctorFacade;

    /**
     * @Author songxl
     * @Description 数据权限CRUD方法
     * @Date 2021/7/21
     * @Param [serviceDataAuthVO]
     * @Return void
     * @MethodName CRUDOperation
     */
    @Transactional
    public boolean CRUDOperation(SoftwareDataAuthVO softwareDataAuthVO) {
        try {
            //1.入参校验
            inputParamCheck(softwareDataAuthVO);
            //2.执行操作
            if (CRUDEnum.ADD.getKey() == softwareDataAuthVO.getType()) {                 //添加
                return addDataAuth(softwareDataAuthVO);
            } else if (CRUDEnum.UPDATE.getKey() == softwareDataAuthVO.getType()) {       //修改
                return updateDataAuth(softwareDataAuthVO);
            } else if (CRUDEnum.DELETE.getKey() == softwareDataAuthVO.getType()) {       //删除
                return deleteDataAuth(softwareDataAuthVO);
            } else if (CRUDEnum.DISBALE.getKey() == softwareDataAuthVO.getType()) {       //启用禁用
                return disableDataAuth(softwareDataAuthVO);
            } else {
                Asserts.fail("操作码错误");
            }
        } catch (Exception e) {
            e.printStackTrace();
            Asserts.fail(e.toString());
        }
        return false;
    }

    /**
     * @Author songxl
     * @Description禁用启用数据权限
     * @Date 2021/7/23
     * @Param [serviceDataAuthVO]
     * @Return boolean
     * @MethodName disableDataAuth
     */
    private boolean disableDataAuth(SoftwareDataAuthVO serviceDataAuthVO) {
        return dataAuthFacade.update(new UpdateWrapper<DataAuth>()
                .set("status", serviceDataAuthVO.getDataAuthVO().getStatus())
                .eq("id", serviceDataAuthVO.getDataAuthVO().getId())
                .eq("is_deleted", IsDeleteEnum.N.getKey()));
    }

    /**
     * @Author songxl
     * @Description修改数据权限
     * @Date 2021/7/23
     * @Param [softwareDataAuthVO]
     * @Return boolean
     * @MethodName updateDataAuth
     */
    private boolean updateDataAuth(SoftwareDataAuthVO softwareDataAuthVO) {

        //1.修改数据权限
        boolean dataAuthUpdate = dataAuthFacade.update(new UpdateWrapper<DataAuth>()
                .set("name", softwareDataAuthVO.getDataAuthVO().getName())
                .eq("id", softwareDataAuthVO.getDataAuthVO().getId())
                .eq("is_deleted", IsDeleteEnum.N.getKey()));
        if (dataAuthUpdate) {
            //2.获取该权限 系统与数据权限信息关联表（sys_service_data_auth）的关联id
            List<Long> serviceDataAuthIDS = softwareDataAuthFacade.list(new QueryWrapper<SoftwareDataAuth>()
                    .eq("data_auth_id", softwareDataAuthVO.getDataAuthVO().getId())
                    .eq("is_deleted", IsDeleteEnum.N.getKey())).stream().map(SoftwareDataAuth::getId).collect(Collectors.toList());
            //3.通过serviceDataAuthIDS 删除sys_role_service_data_auth和sys_service_data_auth和sys_data_auth_detail 对应关系
            if (ListUtil.isNotEmpty(serviceDataAuthIDS)) {
                if (!deleteServiceDataAuth(serviceDataAuthIDS)) {
                    Asserts.fail("数据权限与系统关系删除失败");
                }
            }
            //4.插入新建数据权限与系统的关系
            saveServiceDataAuth(softwareDataAuthVO, softwareDataAuthVO.getDataAuthVO().getId());
        } else {
            Asserts.fail("数据权限修改失败");
        }
        return true;
    }

    /**
     * @Author songxl
     * @Description通过serviceDataAuthIDS删除sys_role_service_data_auth和sys_service_data_auth和sys_data_auth_detail对应关系
     * @Date 2021/7/23
     * @Param [serviceDataAuthIDS]
     * @Return void
     * @MethodName deleteServiceDataAuth
     */
    private boolean deleteServiceDataAuth(List<Long> serviceDataAuthIDS) {
        if (dataAuthDetailFacade.remove(new UpdateWrapper<DataAuthDetail>()
                .in("software_data_auth_id", serviceDataAuthIDS)
                .eq("is_deleted", IsDeleteEnum.N.getKey()))) {
            if (roleSoftwareDataAuthFacade.remove(new UpdateWrapper<RoleSoftwareDataAuth>()
                    .in("software_data_auth_id", serviceDataAuthIDS)
                    .eq("is_deleted", IsDeleteEnum.N.getKey()))) {
                return softwareDataAuthFacade.remove(new UpdateWrapper<SoftwareDataAuth>()
                        .in("id", serviceDataAuthIDS)
                        .eq("is_deleted", IsDeleteEnum.N.getKey()));
            } else {
                Asserts.fail("数据权限与系统、角色关联关系删除失败");
            }
        } else {
            Asserts.fail("系统数据权限与数据权限明细关联关系删除失败");
        }
        return false;
    }

    /**
     * @Author songxl
     * @Description删除数据权限
     * @Date 2021/7/23
     * @Param [serviceDataAuthVO]
     * @Return boolean
     * @MethodName deleteDataAuth
     */
    private boolean deleteDataAuth(SoftwareDataAuthVO serviceDataAuthVO) {
        //1.删除数据权限
        boolean dataAuthdelete = dataAuthFacade.remove(new UpdateWrapper<DataAuth>()
                .eq("id", serviceDataAuthVO.getDataAuthVO().getId())
                .eq("is_deleted", IsDeleteEnum.N.getKey()));
        if (dataAuthdelete) {
            //2.获取该权限 系统与数据权限信息关联表（sys_service_data_auth）的关联id
            List<Long> serviceDataAuthIDS = softwareDataAuthFacade.list(new QueryWrapper<SoftwareDataAuth>()
                    .eq("data_auth_id", serviceDataAuthVO.getDataAuthVO().getId())
                    .eq("is_deleted", IsDeleteEnum.N.getKey())).stream().map(SoftwareDataAuth::getId).collect(Collectors.toList());
            //3.通过serviceDataAuthIDS 删除sys_role_service_data_auth和sys_service_data_auth和sys_data_auth_detail 对应关系
            if (ListUtil.isNotEmpty(serviceDataAuthIDS)) {
                if (deleteServiceDataAuth(serviceDataAuthIDS)) {
                    return true;
                } else {
                    Asserts.fail("数据权限与系统关系删除失败");
                }
            }
        } else {
            Asserts.fail("数据权限删除失败");
        }
        return true;
    }

    /**
     * @Author songxl
     * @Description添加数据权限
     * @Date 2021/7/21
     * @Param [serviceDataAuthVO]
     * @Return void
     * @MethodName addDataAuth
     */
    private boolean addDataAuth(SoftwareDataAuthVO serviceDataAuthVO) {
        //1.插入新建数据权限
        DataAuth dataAuth = new DataAuth();
        //        dataAuth.setCreator(Long.parseLong(SysUserUtils.getCurrentPrincipleID()));
        BeanUtils.copyProperties(serviceDataAuthVO.getDataAuthVO(), dataAuth);
        dataAuth.setGmtCreate(DateUtil.now());
        dataAuth.setCreator("1");
        boolean dataAuthInsert = dataAuthFacade.save(dataAuth);
        if (dataAuthInsert) {
            //2.插入成功获取新生成的数据权限名称编号data_auth_id
            Long dataAuthId = dataAuthFacade.list(new QueryWrapper<DataAuth>()
                    .eq("name", dataAuth.getName())
                    .eq("is_deleted", IsDeleteEnum.N.getKey())).get(0).getId();

            //3.插入新建数据权限与系统的关系
            saveServiceDataAuth(serviceDataAuthVO, dataAuthId);

        } else {
            Asserts.fail("数据权限插入失败");
        }
        return true;

    }

    /**
     * @Author songxl
     * @Description插入新建数据权限与系统的关系
     * @Date 2021/7/23
     * @Param [serviceDataAuthVO, dataAuthId]
     * @Return void
     * @MethodName saveServiceDataAuth
     */
    private void saveServiceDataAuth(SoftwareDataAuthVO serviceDataAuthVO, Long dataAuthId) {
        for (SysSoftwareVO sysServiceVO : serviceDataAuthVO.getDataAuthVO().getSoftwareVOS()) {
            SoftwareDataAuth serviceDataAuth = new SoftwareDataAuth();
            serviceDataAuth.setSoftwareId(sysServiceVO.getId());
            serviceDataAuth.setDataAuthId(dataAuthId);
            serviceDataAuth.setGmtCreate(DateUtil.now());
            serviceDataAuth.setCreator("1");
            //                serviceDataAuth.setCreator(Long.parseLong(SysUserUtils.getCurrentPrincipleID()));
            boolean serviceDataAuthInsert = softwareDataAuthFacade.save(serviceDataAuth);
            if (serviceDataAuthInsert) {
                //1.获取上一步插入时新生成的数据权限名称与系统关联编号service_data_auth _id
                Long serviceDataAuthId = softwareDataAuthFacade.list(new QueryWrapper<SoftwareDataAuth>()
                        .eq("software_id", sysServiceVO.getId())
                        .eq("data_auth_id", dataAuthId)
                        .eq("is_deleted", IsDeleteEnum.N.getKey())).get(0).getId();

                //2.插入数据权限系统关联表与角色管理sys_role_service_data_auth
                for (RoleVO role : serviceDataAuthVO.getRoles()) {
                    RoleSoftwareDataAuth roleServiceDataAuth = new RoleSoftwareDataAuth();
                    roleServiceDataAuth.setRoleId(role.getId());
                    roleServiceDataAuth.setSoftwareDataAuthId(serviceDataAuthId);
                    roleServiceDataAuth.setGmtCreate(DateUtil.now());
                    roleServiceDataAuth.setCreator("1");
                    //                        roleServiceDataAuth.setCreator(Long.parseLong(SysUserUtils.getCurrentPrincipleID()));
                    if (!roleSoftwareDataAuthFacade.save(roleServiceDataAuth)) {
                        Asserts.fail("角色对应服务数据权限关系插入失败");
                    }
                }

                //3.插入数据权限详细信息。sys_data_auth_detail
                for (DataAuthDetailVO dataAuthDetailVO : sysServiceVO.getDataAuthDetailVOS()) {
                    DataAuthDetail dataAuthDetail = new DataAuthDetail();
                    BeanUtils.copyProperties(dataAuthDetailVO, dataAuthDetail);
                    //                        dataAuthDetail.setCreator(Long.parseLong(SysUserUtils.getCurrentPrincipleID()));
                    dataAuthDetail.setGmtCreate(DateUtil.now());
                    dataAuthDetail.setCreator("1");
                    dataAuthDetail.setSoftwareDataAuthId(serviceDataAuthId);
                    if (!dataAuthDetailFacade.save(dataAuthDetail)) {
                        Asserts.fail("数据权限明细插入失败");
                    }
                }

            } else {
                Asserts.fail("服务对应数据权限关系插入失败");
            }
        }
    }

    /**
     * @Author songxl
     * @Description 入参校验
     * @Date 2021/7/21
     * @Param [serviceDataAuthVO]
     * @Return void
     * @MethodName inputParamCheck
     */
    private void inputParamCheck(SoftwareDataAuthVO serviceDataAuthVO) {
        if (serviceDataAuthVO == null) {
            Asserts.fail("入参为空");
        }
        if (serviceDataAuthVO.getType() == 0) {
            Asserts.fail("操作码为空");
        }
        switch (serviceDataAuthVO.getType()) {
            case 1:
                if (StringUtil.isEmpty(serviceDataAuthVO.getDataAuthVO().getName())) {
                    Asserts.fail("数据权限名称为空");
                }
                if (ListUtil.isEmpty(serviceDataAuthVO.getDataAuthVO().getSoftwareVOS())) {
                    Asserts.fail("数据权限对应系统未选择");
                }
                if (ListUtil.isEmpty(serviceDataAuthVO.getDataAuthVO().getSoftwareVOS().get(0).getDataAuthDetailVOS())) {
                    Asserts.fail("系统对应数据权限明细未选择");
                }
                break;
            case 2:
                if (serviceDataAuthVO.getDataAuthVO().getId() == null) {
                    Asserts.fail("数据权限ID为空");
                }
                if (StringUtil.isEmpty(serviceDataAuthVO.getDataAuthVO().getName())) {
                    Asserts.fail("数据权限名称为空");
                }
                if (ListUtil.isEmpty(serviceDataAuthVO.getDataAuthVO().getSoftwareVOS())) {
                    Asserts.fail("数据权限对应系统未选择");
                }
                if (ListUtil.isEmpty(serviceDataAuthVO.getDataAuthVO().getSoftwareVOS().get(0).getDataAuthDetailVOS())) {
                    Asserts.fail("系统对应数据权限明细未选择");
                }
                break;
            case 3:
                if (serviceDataAuthVO.getDataAuthVO().getId() == null) {
                    Asserts.fail("数据权限ID为空");
                }
                break;
            case 5:
                if (serviceDataAuthVO.getDataAuthVO().getId() == null) {
                    Asserts.fail("数据权限ID为空");
                }
                if (serviceDataAuthVO.getDataAuthVO().getStatus() == null) {
                    Asserts.fail("status为空");
                }
                break;
            default:

                break;
        }

    }

    /**
     * @Author songxl
     * @Description通过数据权限id和角色id获取数据权限详情
     * @Date 2021/7/22
     * @Param [serviceDataAuthVO]
     * @Return com.lantone.security.dto.DataAuthDTO
     * @MethodName getDataAuthById
     */
    public DataAuthDTO getDataAuthById(SoftwareDataAuthVO serviceDataAuthVO) {
        if (serviceDataAuthVO.getDataAuthVO().getId() == null) {
            Asserts.fail("数据权限ID为空");
        }
        DataAuthDTO dataAuthDTO = new DataAuthDTO();
        List<SoftwareRoleDataAuthDetailDTO> serviceRoleDataAuthDetailDTOS = this.baseMapper.getDataAuthById(serviceDataAuthVO);
        if (ListUtil.isNotEmpty(serviceRoleDataAuthDetailDTOS)) {
            //1.添加数据权限信息
            dataAuthDTO.setId(serviceRoleDataAuthDetailDTOS.get(0).getDataAuthId());
            dataAuthDTO.setName(serviceRoleDataAuthDetailDTOS.get(0).getDataAuthName());
            dataAuthDTO.setStatus(serviceRoleDataAuthDetailDTOS.get(0).getStatus());
            //2.获取系统信息
            Set<Long> serviceSet = serviceRoleDataAuthDetailDTOS.stream().map(SoftwareRoleDataAuthDetailDTO::getSoftwareId).collect(Collectors.toSet());
            List<SysServiceDTO> sysServiceDTOS = new ArrayList<>();
            serviceSet.stream().forEach(id -> {
                //3.添加服务信息
                SysServiceDTO sysServiceDTO = new SysServiceDTO();
                sysServiceDTO.setId(id);
                sysServiceDTO.setName(ServiceEnum.getName(id.intValue()));
                //4.1添加数据权限详情列表
                //                List<ServiceRoleDataAuthDetailDTO> oneServiceRoleDataAuthDetailDTOS = serviceRoleDataAuthDetailDTOS.stream()
                //                        .filter(serviceRoleDataAuthDetailDTO -> serviceRoleDataAuthDetailDTO.getServiceId() == id).collect(Collectors.toList());
                List<SoftwareRoleDataAuthDetailDTO> oneServiceRoleDataAuthDetailDTOS = this.baseMapper.getOneSoftwareRoleDataAuthDetail(id, serviceDataAuthVO.getDataAuthVO().getId());

                List<DataAuthDetailDTO> dataAuthDetailDTOS = new ArrayList<>();
                //4.2.1是否获取当前组织-科室信息 dataAuthDetailDTOS去重
                oneServiceRoleDataAuthDetailDTOS.stream().forEach(serviceRoleDataAuthDetailDTO -> {
                    DataAuthDetailDTO dataAuthDetailDTO = new DataAuthDetailDTO();
                    dataAuthDetailDTO.setId(serviceRoleDataAuthDetailDTO.getDetail());
                    dataAuthDetailDTO.setDataType(serviceRoleDataAuthDetailDTO.getDataType());
                    dataAuthDetailDTO.setDetailType(serviceRoleDataAuthDetailDTO.getDetailType());
                    dataAuthDetailDTO.setDetailId(serviceRoleDataAuthDetailDTO.getDetailId());
                    if (DataAuthDataTypeEnum.CUSTOMIZE.getKey() == serviceRoleDataAuthDetailDTO.getDataType()
                            && serviceRoleDataAuthDetailDTO.getDetailType() != null
                            && DataAuthDetailTypeEnum.DEPT.getKey() == serviceRoleDataAuthDetailDTO.getDetailType()) {
                        //4.2.3selectFalg为true时查询组织-科室信息
                        getDeptDetial(dataAuthDetailDTO);
                    }
                    dataAuthDetailDTOS.add(dataAuthDetailDTO);
                });
                sysServiceDTO.setDataAuthDetailDTOS(dataAuthDetailDTOS);

                //5.添加角色列表
                List<RoleDTO> roleDTOS = new ArrayList<>();
                Map<Long, String> roleMap = new HashMap<>();
                serviceRoleDataAuthDetailDTOS.stream().forEach(serviceRoleDataAuthDetailDTO -> {
                    roleMap.put(serviceRoleDataAuthDetailDTO.getRoleId(), serviceRoleDataAuthDetailDTO.getRoleName());
                });
                for (Long roleID : roleMap.keySet()) {
                    RoleDTO role = new RoleDTO();
                    role.setId(roleID);
                    role.setName(roleMap.get(roleID));
                    roleDTOS.add(role);
                }
                sysServiceDTO.setRoles(roleDTOS);
                sysServiceDTOS.add(sysServiceDTO);
            });
            dataAuthDTO.setSysServiceDTOS(sysServiceDTOS);
        } else {
            Asserts.fail("请检查角色id或数据权限id");
        }

        return dataAuthDTO;
    }

    /**
     * @Author songxl
     * @Description 获取组织-科室信息以及自定义科室
     * @Date 2021/7/22
     * @Param [dataAuthDetailDTO]
     * @Return void
     * @MethodName getDeptDetial
     */
    public void getDeptDetial(DataAuthDetailDTO dataAuthDetailDTO) {
        //1.获取医院信息
        HospitalDTO hospitalDTO = new HospitalDTO();
        Hospital hospital = hospitalFacade.list(new QueryWrapper<Hospital>()
                .eq("id", 1l)//###
                .eq("type", "0")//###
                .eq("is_deleted", IsDeleteEnum.N.getKey())).get(0);
        BeanUtils.copyProperties(hospital, hospitalDTO);
        //获取当前登录用户的组织信息
        List<HospitalDTO> hospitalDTOS = getHospitalInfo(hospitalDTO);
        getAllHospitalDTO(hospitalDTOS);
        //2.获取科室信息
        getDeptInfo(hospitalDTO, dataAuthDetailDTO);
        dataAuthDetailDTO.setHospitalDTOS(hospitalDTO);


    }

    /**
     * @Author songxl
     * @Description获取对应医院的科室信息
     * @Date 2021/7/23
     * @Param [hospitalDTO, dataAuthDetailDTOS]
     * @Return void
     * @MethodName getDeptInfo
     */
    private void getDeptInfo(HospitalDTO hospitalDTO, DataAuthDetailDTO dataAuthDetailDTO) {
        if (hospitalDTO != null) {
            if (ListUtil.isNotEmpty(hospitalDTO.getHospitalDTOS())) {
                for (HospitalDTO oneHospitalDTO : hospitalDTO.getHospitalDTOS()) {
                    //虚拟医院继续递归遍历到真实医院
                    getDeptInfo(oneHospitalDTO, dataAuthDetailDTO);
                }
                //真实医院：获取科室
            } else if ((HospitalTypeEnum.SINGLE_HOSPITAL.getKey() + "").equals(hospitalDTO.getType())) {
                List<Dept> depts = deptFacade.list(new QueryWrapper<Dept>()
                        .eq("hospital_id", hospitalDTO.getId())
                        .eq("is_deleted", IsDeleteEnum.N.getKey()));
                List<DeptDTO> deptDTOS = new ArrayList<>();
                if (ListUtil.isNotEmpty(depts)) {
                    for (Dept dept : depts) {
                        DeptDTO deptDTO = new DeptDTO();
                        BeanUtils.copyProperties(dept, deptDTO);
                        deptDTOS.add(deptDTO);
                        //该医院和该医院的科室 自定义关联关系为true
                        if (dataAuthDetailDTO.getDetailId() == deptDTO.getId()) {
                            hospitalDTO.setRelationStatus(true);
                            deptDTO.setRelationStatus(true);
                        }
                    }
                }
                hospitalDTO.setDepts(deptDTOS);
            }
        }

    }

    /**
     * @Author songxl
     * @Description 获取医院对应的子医院信息
     * @Date 2021/7/23
     * @Param [hospitalDTOS]
     * @Return void
     * @MethodName getAllHospitalDTO
     */
    private void getAllHospitalDTO(List<HospitalDTO> hospitalDTOS) {
        if (ListUtil.isNotEmpty(hospitalDTOS)) {
            for (HospitalDTO tempHospitalDTO : hospitalDTOS) {
                getAllHospitalDTO(getHospitalInfo(tempHospitalDTO));
            }
        }
    }

    /**
     * @Author songxl
     * @Description获取医院信息
     * @Date 2021/7/22
     * @Param [parentId]
     * @Return java.util.List<com.lantone.common.dto.HospitalDTO>
     * @MethodName getHospitalInfo
     */
    private List<HospitalDTO> getHospitalInfo(HospitalDTO hospitalDTO) {
        List<HospitalDTO> hospitalDTOS = new ArrayList<>();
        if (hospitalDTO.getId() != null) {
            List<Hospital> hospitals = hospitalFacade.list(new QueryWrapper<Hospital>()
                    .eq("parent_id", hospitalDTO.getId())
                    .eq("is_deleted", IsDeleteEnum.N.getKey()));
            hospitals.stream().forEach(hospital -> {
                HospitalDTO tempHospitalDTO = new HospitalDTO();
                BeanUtils.copyProperties(hospital, tempHospitalDTO);
                hospitalDTOS.add(tempHospitalDTO);
            });
            hospitalDTO.setHospitalDTOS(hospitalDTOS);
        }
        return hospitalDTOS;
    }

    /**
     * @Author songxl
     * @Description通过医院id或科室id获取医生列表
     * @Date 2021/7/27
     * @Param [hospitalVO]
     * @Return com.baomidou.mybatisplus.core.metadata.IPage<com.lantone.common.dto.DoctorDTO>
     * @MethodName getDoctorPage
     */
    public IPage<DoctorDTO> getDoctorPage(DoctorVO doctorVO) {
        return doctorFacade.getBaseMapper().getDoctorPage(doctorVO);
    }

    /**
     * @Author songxl
     * @Description获取数据权限列表
     * @Date 2021/7/27
     * @Param [dataAuthVO]
     * @Return com.baomidou.mybatisplus.core.metadata.IPage<com.lantone.common.dto.DataAuthDTO>
     * @MethodName getDataAuthPage
     */
    public IPage<DataAuthDTO> getDataAuthPage(DataAuthVO dataAuthVO) {
        return dataAuthFacade.getBaseMapper().getDataAuthPage(dataAuthVO);
    }
}
