package com.lantone.security.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.lantone.common.dto.MenuDTO;
import com.lantone.common.exception.Asserts;
import com.lantone.common.util.DateUtil;
import com.lantone.common.util.EntityUtil;
import com.lantone.common.util.ListUtil;
import com.lantone.common.util.StringUtil;
import com.lantone.common.vo.MenuVO;
import com.lantone.common.vo.RoleSoftwareMenuVo;
import com.lantone.dblayermbg.entity.Menu;
import com.lantone.dblayermbg.entity.MenuResource;
import com.lantone.dblayermbg.entity.Resource;
import com.lantone.dblayermbg.entity.RoleSoftwareMenu;
import com.lantone.dblayermbg.entity.SoftwareMenu;
import com.lantone.dblayermbg.facade.MenuFacade;
import com.lantone.dblayermbg.facade.MenuResourceFacade;
import com.lantone.dblayermbg.facade.ResourceFacade;
import com.lantone.dblayermbg.facade.RoleSoftwareMenuFacade;
import com.lantone.dblayermbg.facade.SoftwareMenuFacade;
import com.lantone.security.enums.CRUDEnum;
import com.lantone.security.enums.IsDeleteEnum;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @ClassName: FuncManagementFacade
 * @Description: 功能管理API
 * @Author songxl
 * @Date 2021/7/28
 * @Version 1.0
 */
@Component
public class FuncManagementFacade {
    @Autowired
    private RoleSoftwareMenuFacade roleSoftwareMenuFacade;
    @Autowired
    private MenuFacade menuFacade;
    @Autowired
    private ResourceFacade resourceFacade;
    @Autowired
    private MenuResourceFacade menuResourceFacade;
    @Autowired
    private SoftwareMenuFacade softwareMenuFacade;

    /**
     * @param roleSoftwareMenuVo
     * @Method getMenus
     * @Author songxl
     * @Version 1.0
     * @Description获取功能菜单树
     * @Return java.util.List<com.lantone.common.dto.MenuDTO>
     * @Date 2021/7/28
     */
    public List<MenuDTO> getMenusTree(RoleSoftwareMenuVo roleSoftwareMenuVo) {

        //1.入参校验
        inputParamCheck(roleSoftwareMenuVo);
        List<MenuDTO> menuList = null;
        //2.1获取菜单列表
        menuList = roleSoftwareMenuFacade.getBaseMapper().getMenus(roleSoftwareMenuVo);
        //3.整理菜单
        if (ListUtil.isNotEmpty(menuList)) {
            Map<Long, List<MenuDTO>> menuMap = EntityUtil.makeEntityListMap(menuList, "parentId");
            List<MenuDTO> menuRes = menuMap.get(0L);
            for (MenuDTO bean : menuRes) {
                getSonMenu(bean, menuMap);
            }
            return menuRes;
        } else {
            Asserts.fail("菜单列表为空");
        }
        return null;
    }

    /**
     * @param roleSoftwareMenuVo
     * @Method inputParamCheck
     * @Author songxl
     * @Version 1.0
     * @Description入参校验
     * @Return void
     * @Date 2021/7/28
     */
    private void inputParamCheck(RoleSoftwareMenuVo roleSoftwareMenuVo) {
        if (roleSoftwareMenuVo == null) {
            Asserts.fail("入参为空");
        }
        if (ListUtil.isEmpty(roleSoftwareMenuVo.getRoles())) {
            Asserts.fail("角色列表为空");
        }
        if (roleSoftwareMenuVo.getSoftwareId() == null) {
            Asserts.fail("服务id为空");
        }
    }

    /**
     * @param menuVO
     * @Method inputParamCheck
     * @Author songxl
     * @Version 1.0
     * @Description入参校验
     * @Return void
     * @Date 2021/7/28
     */
    private void inputParamCheck(MenuVO menuVO) {
        if (menuVO == null) {
            Asserts.fail("入参为空");
        }
        if (menuVO.getOpeType() == 0) {
            Asserts.fail("操作码为空");
        }
        switch (menuVO.getOpeType()) {
            case 1:
                if (StringUtil.isEmpty(menuVO.getName())) {
                    Asserts.fail("功能菜单名称为空");
                }
                if (menuVO.getParentId() == null) {
                    Asserts.fail("功能菜单父级ID为空");
                }
                if (ListUtil.isEmpty(menuVO.getSoftwares())) {
                    Asserts.fail("功能菜单所属服务ID列表为空");
                }
                if (menuVO.getType() == null) {
                    Asserts.fail("功能菜单类型为空");
                }
                break;
            case 2:
                if (menuVO.getId() == null) {
                    Asserts.fail("功能菜单ID为空");
                }
                if (StringUtil.isEmpty(menuVO.getName())) {
                    Asserts.fail("功能菜单名称为空");
                }
                if (menuVO.getParentId() == null) {
                    Asserts.fail("功能菜单父级ID为空");
                }
                if (ListUtil.isEmpty(menuVO.getSoftwares())) {
                    Asserts.fail("功能菜单所属服务ID列表为空");
                }
                if (menuVO.getType() == null) {
                    Asserts.fail("功能菜单类型为空");
                }
                break;
            case 3:
                if (menuVO.getId() == null) {
                    Asserts.fail("功能菜单ID为空");
                }
                break;
            case 5:
                if (menuVO.getId() == null) {
                    Asserts.fail("功能菜单ID为空");
                }
                if (menuVO.getStatus() == null) {
                    Asserts.fail("status为空");
                }
                break;
            default:
                break;
        }
    }


    /**
     * 递归获取菜单结构
     *
     * @param menu    当前菜单
     * @param menuMap 菜单集
     * @return 菜单结构
     */
    public List<MenuDTO> getSonMenu(MenuDTO menu, Map<Long, List<MenuDTO>> menuMap) {
        List<MenuDTO> sonMenu = new ArrayList<>();
        List<MenuDTO> menuList = menuMap.get(menu.getId());
        if (ListUtil.isNotEmpty(menuList)) {
            menu.setSonMenu(menuList);
            for (MenuDTO bean : menuList) {
                getSonMenu(bean, menuMap);
            }
        }
        return sonMenu;
    }

    /**
     * @param menuVO
     * @Method addMenu
     * @Author songxl
     * @Version 1.0
     * @Description添加功能菜单
     * @Return java.lang.Boolean
     * @Date 2021/7/28
     */
    private Boolean addMenu(MenuVO menuVO) {
        //1.插入sys_menu
        //1.1插入sys_menu
        Menu menu = new Menu();
        BeanUtils.copyProperties(menuVO, menu);
        menu.setGmtCreate(DateUtil.now());
        if (menuFacade.save(menu)) {
            //1.2插入成功获取 menuID
            Long menuID = menuFacade.list(new QueryWrapper<Menu>()
                    .eq("parent_id", menuVO.getParentId())
                    .eq("name", menuVO.getName())
                    .eq("type", menuVO.getType())
                    .eq("is_deleted", IsDeleteEnum.N.getKey())).get(0).getId();
            //插入sys_resource、sys_menu_resource
            return saveMenuResource(menuVO, menuID);
        } else {
            Asserts.fail("菜单详情插入失败");
        }
        return false;
    }

    /**
     * @param menuVO
     * @Method saveMenuResource
     * @Author songxl
     * @Version 1.0
     * @Description
     * @Return void
     * @Date 2021/7/28
     */
    private boolean saveMenuResource(MenuVO menuVO, Long menuID) {
        //2.3 判断该功能菜单是否有对应权限
        if (ListUtil.isNotEmpty(menuVO.getPermissions())) {
            menuVO.getPermissions().stream().forEach(permission -> {
                //2.4 有权限 则插入sys_resource
                Resource resource = new Resource();
                resource.setUrl(permission);
                resource.setGmtCreate(DateUtil.now());
                if (resourceFacade.save(resource)) {
                    //2.5 插入成功获取对应的 resourceID
                    Long resourceID = resourceFacade.list(new QueryWrapper<Resource>()
                            .select("id")
                            .eq("url", resource.getUrl())
                            .eq("is_deleted", IsDeleteEnum.N.getKey())).get(0).getId();
                    //2.6 插入菜单和功能对应关系
                    MenuResource menuResource = new MenuResource();
                    menuResource.setMenuId(menuID);
                    menuResource.setResourceId(resourceID);
                    if (menuResourceFacade.save(menuResource)) {
                    } else {
                        Asserts.fail("菜单与资源权限插入失败");
                    }
                } else {
                    Asserts.fail("资源权限插入失败");
                }
            });
        }
        menuVO.getSoftwares().stream().forEach(softwareId -> {
            //3.插入超级管理员对应系统 sys_software_menu和sys_role_software_menu
            savaRoleSoftwareMenu(menuID, softwareId, 1l);
        });
        return true;
    }

    /**
     * @param menuID
     * @param softwareId
     * @param roleID
     * @Method savaRoleSoftwareMenu
     * @Author songxl
     * @Version 1.0
     * @Description插入超级管理员对应系统 sys_software_menu和sys_role_software_menu
     * @Return void
     * @Date 2021/7/28
     */
    private void savaRoleSoftwareMenu(Long menuID, Long softwareId, Long roleID) {
        SoftwareMenu softwareMenu = new SoftwareMenu();
        softwareMenu.setMenuId(menuID);
        softwareMenu.setSoftwareId(softwareId);
        if (softwareMenuFacade.save(softwareMenu)) {
            Long softwareMenuId = softwareMenuFacade.list(new QueryWrapper<SoftwareMenu>()
                    .eq("software_id", softwareMenu.getSoftwareId())
                    .eq("menu_id", softwareMenu.getMenuId())
                    .eq("is_deleted", IsDeleteEnum.N.getKey())).get(0).getId();
            RoleSoftwareMenu roleSoftwareMenu = new RoleSoftwareMenu();
            roleSoftwareMenu.setRoleId(roleID);
            roleSoftwareMenu.setSoftwareMenuId(softwareMenuId);
            if (roleSoftwareMenuFacade.save(roleSoftwareMenu)) {

            } else {
                Asserts.fail("角色与服务菜单关系插入失败");
            }
        } else {
            Asserts.fail("服务与菜单对应关系插入失败");
        }
    }

    /**
     * @param menuVO
     * @Method CRUDOperation
     * @Author songxl
     * @Version 1.0
     * @Description功能菜单CRUD方法
     * @Return java.lang.Boolean
     * @Date 2021/7/28
     */
    public Boolean CRUDOperation(MenuVO menuVO) {

        try {
            //1.入参校验
            inputParamCheck(menuVO);
            //2.执行操作
            if (CRUDEnum.ADD.getKey() == menuVO.getOpeType()) {                 //添加
                return addMenu(menuVO);
            } else if (CRUDEnum.UPDATE.getKey() == menuVO.getOpeType()) {       //修改
                return updateDataAuth(menuVO);
            } else if (CRUDEnum.DELETE.getKey() == menuVO.getOpeType()) {       //删除
                return deleteDataAuth(menuVO);
            } else if (CRUDEnum.DISBALE.getKey() == menuVO.getOpeType()) {       //启用禁用
                return disableDataAuth(menuVO);
            } else {
                Asserts.fail("操作码错误");
            }
        } catch (Exception e) {
            e.printStackTrace();
            Asserts.fail(e.toString());
        }
        return false;
    }

    /**
     * @param menuVO
     * @Method disableDataAuth
     * @Author songxl
     * @Version 1.0
     * @Description启用禁用功能菜单
     * @Return java.lang.Boolean
     * @Date 2021/7/28
     */
    private Boolean disableDataAuth(MenuVO menuVO) {
        return menuFacade.update(new UpdateWrapper<Menu>()
                .set("status",menuVO.getStatus())
                .eq("id",menuVO.getId())
                .eq("is_deleted", IsDeleteEnum.N.getKey()));
    }

    /**
     * @param menuVO
     * @Method deleteDataAuth
     * @Author songxl
     * @Version 1.0
     * @Description删除功能菜单
     * @Return java.lang.Boolean
     * @Date 2021/7/28
     */
    private Boolean deleteDataAuth(MenuVO menuVO) {
        //1.删除原来menu关联插入的数据
        if(menuFacade.remove(new UpdateWrapper<Menu>()
                .eq("id",menuVO.getId())
                .eq("is_deleted", IsDeleteEnum.N.getKey()))){
            return deleteMenuJoinDate(menuVO, 1l);
        }else {
            Asserts.fail("菜单详情删除失败");
        }
        return false;
    }

    /**
     * @param menuVO
     * @Method updateDataAuth
     * @Author songxl
     * @Version 1.0
     * @Description修改功能菜单
     * @Return java.lang.Boolean
     * @Date 2021/7/28
     */
    private Boolean updateDataAuth(MenuVO menuVO) {
        //1.删除原来menu关联插入的数据
        if(deleteMenuJoinDate(menuVO, 1l)){
            //2.修改sys_menu 菜单表
            Menu menu = new Menu();
            BeanUtils.copyProperties(menuVO, menu);
           if( menuFacade.update(new UpdateWrapper<Menu>()
                   .set("name",menu.getName())
                   .set("type",menu.getType())
                   .set("url",menu.getUrl())
                   .set("order_no",menu.getOrderNo())
                   .set("menu_describe",menu.getMenuDescribe())
                   .set("remark",menu.getRemark())
                   .set("status",menu.getStatus())
                   .eq("id", menu.getId()))){
               //3.重新插入上面删除的数据
               return saveMenuResource(menuVO,menuVO.getId());
           }
        }
        return false;
    }
    /**
     * @Method deleteMenuJoinDate
     * @Author songxl
     * @Version  1.0
     * @Description清除原菜单关联的数据
     * @param menuVO
     * @param roleId
     * @Return boolean
     * @Date 2021/7/29
     */
    private boolean deleteMenuJoinDate(MenuVO menuVO, Long roleId) {

        //2.1删除sys_resource、sys_menu_resource原先对应menu的资源权限
        List<MenuResource> menuResources = menuResourceFacade.list(new QueryWrapper<MenuResource>()
                .eq("menu_id", menuVO.getId())
                .eq("is_deleted", IsDeleteEnum.N.getKey()));
        //2.1有对应的资源权限
        if (ListUtil.isNotEmpty(menuResources) && menuResources.get(0).getResourceId() != null) {
            if (menuResourceFacade.remove(new UpdateWrapper<MenuResource>()
                    .eq("id", menuResources.get(0).getId())
                    .eq("is_deleted", IsDeleteEnum.N.getKey()))) {
                if (resourceFacade.remove(new UpdateWrapper<Resource>()
                        .eq("id", menuResources.get(0).getResourceId())
                        .eq("is_deleted", IsDeleteEnum.N.getKey()))) {
                } else {
                    Asserts.fail("原菜单对应资源详情删除失败");
                }

            } else {
                Asserts.fail("原菜单对应资源关联关系删除失败");
            }
        }
        //2.2删除超级管理员该菜单的管理sys_software_menu、sys_role_software_menu
        List<Long> softwareMenuIds = roleSoftwareMenuFacade.list(new QueryWrapper<RoleSoftwareMenu>()
                .eq("role_id", roleId)
                .eq("is_deleted", IsDeleteEnum.N.getKey())).stream().map(RoleSoftwareMenu::getSoftwareMenuId).collect(Collectors.toList());
        List<SoftwareMenu> softwareMenus = softwareMenuFacade.getBaseMapper().getSoftwareMenu(menuVO.getId(), softwareMenuIds);
        //如果改菜单没有超级管理员对应关系
        if(ListUtil.isEmpty(softwareMenus)){
            return true;
        }
        //有删除
        if (roleSoftwareMenuFacade.remove(new UpdateWrapper<RoleSoftwareMenu>()
                .in("software_menu_id", softwareMenus.stream().map(SoftwareMenu::getId).collect(Collectors.toList()))
                .eq("is_deleted", IsDeleteEnum.N.getKey()))) {
            if (softwareMenuFacade.remove(new UpdateWrapper<SoftwareMenu>()
                    .in("id", softwareMenus.stream().map(SoftwareMenu::getId).collect(Collectors.toList()))
                    .eq("is_deleted", IsDeleteEnum.N.getKey()))) {
                return true;
            }else {
                Asserts.fail("服务菜单关系清除失败");
            }
        }else {
            Asserts.fail("角色对应服务菜单关系清除失败");
        }
        return false;
    }

    /**
     * @Method getMenuById
     * @Author songxl
     * @Version  1.0
     * @Description获取菜单详情
     * @param menuVO
     * @Return com.lantone.common.dto.MenuDTO
     * @Date 2021/7/29
     */
    public MenuDTO getMenuById(MenuVO menuVO) {
        return menuFacade.getBaseMapper().getMenuById(menuVO);
    }
}
